@@ -1,4 +1,3 @@
-use crate::Vec;
 use ark_crypto_primitives::Error;
 use ark_ff::{bytes::ToBytes, fields::PrimeField};
 
@@ -9,9 +8,9 @@ pub mod constraints;
 #[cfg(feature = "r1cs")]
 pub use constraints::*;
 
-pub trait Set<F: PrimeField>: Sized {
+pub trait Set<F: PrimeField, const M: usize>: Sized {
 	type Private: Clone + Default;
 
-	fn generate_secrets<T: ToBytes>(target: &T, set: &Vec<F>) -> Result<Self::Private, Error>;
-	fn check<T: ToBytes>(target: &T, private: &Self::Private) -> Result<bool, Error>;
+	fn generate_secrets<T: ToBytes>(target: &T, set: &[F; M]) -> Result<Self::Private, Error>;
+	fn check<T: ToBytes>(target: &T, set: &[F; M], private: &Self::Private) -> Result<bool, Error>;
 }
